@@ -133,13 +133,8 @@ class TestChainMapper(unittest.TestCase):
     ed.SetResidueNumber(r, mol.ResNum(r.GetNumber().GetNum() + 42))
     self.assertRaises(Exception, ChainMapper, tmp_ent, resnum_alignments=True)
 
-    # chain B has a missing Valine... set pep_gap_thr to 0.0 should give an
-    # additional chem group
-    mapper = ChainMapper(ent, pep_gap_thr=0.0)
-    self.assertEqual(len(mapper.chem_groups), 4)
-
     # introduce single point mutation in ent... increasing pep_seqid_thr to 100
-    # should give an additional chem group too
+    # should give an additional chem group
     mapper = ChainMapper(ent, pep_seqid_thr=100.)
     self.assertEqual(len(mapper.chem_groups), 3)
     tmp_ent = ent.Copy()
@@ -166,9 +161,6 @@ class TestChainMapper(unittest.TestCase):
     mapper = ChainMapper(tmp_ent, nuc_seqid_thr=100.)
     self.assertEqual(len(mapper.polynuc_seqs), 3)
     self.assertEqual(len(mapper.chem_groups), 4)
-    mapper = ChainMapper(tmp_ent, nuc_gap_thr=0.0)
-    self.assertEqual(len(mapper.polynuc_seqs), 3)
-    self.assertEqual(len(mapper.chem_groups), 4)
 
   def test_chem_mapping(self):
 
@@ -261,6 +253,9 @@ class TestChainMapper(unittest.TestCase):
     greedy_lddt_res = mapper.GetlDDTMapping(mdl, strategy="greedy_block")
     self.assertEqual(greedy_lddt_res.mapping, [['X', 'Y'],[None],['Z']])
 
+    heuristic_lddt_res = mapper.GetlDDTMapping(mdl, strategy="heuristic")
+    self.assertEqual(heuristic_lddt_res.mapping, [['X', 'Y'],[None],['Z']])
+
 
     # QS score based chain mappings
     naive_qsscore_res = mapper.GetQSScoreMapping(mdl, strategy="naive")
@@ -275,31 +270,37 @@ class TestChainMapper(unittest.TestCase):
     greedy_qsscore_res = mapper.GetQSScoreMapping(mdl, strategy="greedy_block")
     self.assertEqual(naive_qsscore_res.mapping, [['X', 'Y'],[None],['Z']])
 
+    heuristic_qsscore_res = mapper.GetQSScoreMapping(mdl, strategy="heuristic")
+    self.assertEqual(heuristic_qsscore_res.mapping, [['X', 'Y'],[None],['Z']])
+
 
     # rigid chain mappings
-    greedy_rigid_res = mapper.GetRigidMapping(mdl, strategy="greedy_single_gdtts")
+    greedy_rigid_res = mapper.GetRMSDMapping(mdl, strategy="naive")
     self.assertEqual(greedy_rigid_res.mapping, [['X', 'Y'],[None],['Z']])
 
-    greedy_rigid_res = mapper.GetRigidMapping(mdl, strategy="greedy_iterative_gdtts")
+    greedy_rigid_res = mapper.GetRMSDMapping(mdl, strategy="greedy_iterative")
     self.assertEqual(greedy_rigid_res.mapping, [['X', 'Y'],[None],['Z']])
 
-    greedy_rigid_res = mapper.GetRigidMapping(mdl, strategy="greedy_single_rmsd")
+    greedy_rigid_res = mapper.GetRMSDMapping(mdl, strategy="greedy_single")
     self.assertEqual(greedy_rigid_res.mapping, [['X', 'Y'],[None],['Z']])
 
-    greedy_rigid_res = mapper.GetRigidMapping(mdl, strategy="greedy_iterative_rmsd")
-    self.assertEqual(greedy_rigid_res.mapping, [['X', 'Y'],[None],['Z']])
+    heuristic_rigid_res = mapper.GetRMSDMapping(mdl, strategy="heuristic")
+    self.assertEqual(heuristic_rigid_res.mapping, [['X', 'Y'],[None],['Z']])
+
+    # the default chain mapping
+    default_res = mapper.GetMapping(mdl)
 
     # test flat mapping functionality of MappingResult
-    flat_map = greedy_rigid_res.GetFlatMapping()
+    flat_map = default_res.GetFlatMapping()
     self.assertEqual(len(flat_map), 3)
-    self.assertEqual(flat_map[greedy_rigid_res.chem_groups[0][0]], 'X')
-    self.assertEqual(flat_map[greedy_rigid_res.chem_groups[0][1]], 'Y')
-    self.assertEqual(flat_map[greedy_rigid_res.chem_groups[2][0]], 'Z')
-    flat_map = greedy_rigid_res.GetFlatMapping(mdl_as_key=True)
+    self.assertEqual(flat_map[default_res.chem_groups[0][0]], 'X')
+    self.assertEqual(flat_map[default_res.chem_groups[0][1]], 'Y')
+    self.assertEqual(flat_map[default_res.chem_groups[2][0]], 'Z')
+    flat_map = default_res.GetFlatMapping(mdl_as_key=True)
     self.assertEqual(len(flat_map), 3)
-    self.assertEqual(greedy_rigid_res.chem_groups[0][0], flat_map['X'])
-    self.assertEqual(greedy_rigid_res.chem_groups[0][1], flat_map['Y'])
-    self.assertEqual(greedy_rigid_res.chem_groups[2][0], flat_map['Z'])
+    self.assertEqual(default_res.chem_groups[0][0], flat_map['X'])
+    self.assertEqual(default_res.chem_groups[0][1], flat_map['Y'])
+    self.assertEqual(default_res.chem_groups[2][0], flat_map['Z'])
 
     # test Align function of ChainMapper
     _, mdl_polypep_seqs, mdl_polynuc_seqs = mapper.ProcessStructure(mdl)
